package Fairness;

import java.io.File;
import java.io.IOException;

/**
 * FairnessSimulation is the entry point of the simulation software.
 * <p/>
 * FairnessSimulation takes two parameters. The first parameter is the input file or directory. If
 * the input is a file, it is processed as a single simulation. If the first parameter is a
 * directory, then FairnessSimulation will scan that directory for all files ending in ".txt" and
 * run a single simulation for each file, one at a time. The format of the input file is
 * described in the user guide.
 * <p/>
 * The second parameter is the output directory. All of the outputs from the simulation will be
 * placed there. The user guide explains what files are generated for a given simulation, but the
 * name of the simulation file (without its .txt extension) is the name of the simulation, and all
 * output files will start with the name of the simulation followed by a dash.
 *
 * @author Jason Winnebeck
 */
public class FairnessSimulation {
    public static void main( String[] args ) throws SimulationException, IOException {
        if ( args.length != 2 ) {
            System.err.println( "Exactly 2 arguments are required." );
            printUsage();
            System.exit( 1 );
        }

        File input = new File( args[0] );
        File outputDir = new File( args[1] );

        if ( input.isDirectory() ) {
            for ( File file : input.listFiles() ) {
                if ( file.isFile() && file.getName().endsWith( ".txt" ) )
                    runSim( file, outputDir );
            }
        } else if ( input.isFile() ) {
            runSim( input, outputDir );
        } else {
            System.err.println( "Input file " + input.getAbsolutePath() + " must exist." );
            printUsage();
            System.exit( 2 );
        }
    }

    /**
     * Runs a single simulation with the given file as input and the given directory as the output
     * location.
     *
     * @param inputFile non-null File pointing to a valid file
     * @param outputDir non-null File pointing to a valid directory
     *
     * @throws IOException If there are errors reading or writing input and output files
     */
    @SuppressWarnings( { "ErrorNotRethrown" } )
    private static void runSim( File inputFile, File outputDir )
        throws IOException {
        long startTime = System.currentTimeMillis();
        System.out.println( "Running simulation with " + inputFile.getAbsolutePath() );
        try {
            SimulationRunner runner = SimulationBuilder.buildSimulation( inputFile, outputDir );

            runner.runSimulation();
        } catch ( SimulationException e ) {
            printSimError( inputFile, e );
        } catch ( AssertionError e ) {
            printSimError( inputFile, e );
        }
        long diff = ( System.currentTimeMillis() - startTime ) / 1000;
        System.out.println( "  Finished in " + diff + " seconds" );
    }

    /**
     * Prints an error message saying that the Simulation's execution failed.
     */
    private static void printSimError( File inputFile, Throwable e ) {
        System.out.println( "Running simulation input " + inputFile.getAbsolutePath() +
                            " failed." );
        e.printStackTrace();
    }

    /**
     * Prints usage information for the class.
     */
    private static void printUsage() {
        System.err.println( "java Fairness.FairnessSimulation <input> <output dir>" );
        System.err.println( " <input> may refer to a file, or a directory." );
        System.err.println( " If it refers to a directory, all .txt files are taken as inputs" );
    }
}
