package Fairness.Nodes.NodeBehaviors;

import Fairness.Nodes.Node;
import Fairness.Nodes.NodePacket;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * AncestorRatingBehavior keeps track of confidence values and increases/decreases 
 * the confidence levels of nodes for sending and receiving the packets respectively.
 * When a packet is received, the child increases the confidence of all the nodes in the
 * path to the root and similarly decreases if the packet is denied.
 *
 * @author Jason Winnebeck
 */
public class AncestorRatingBehavior implements NodeBehavior {
    private final Map<Node, Confidence> conf = new HashMap<Node, Confidence>();
    private double threshold = -Double.MAX_VALUE;

    /**
     * Allows setting of a single behavior-specific parameter.
     *
     * @param param non-null String
     */
    public void setParameter( String param ) {
        threshold = Double.parseDouble( param );
    }

    /**
     * Asks the node behavior if the given Node is allowed to be a child of the Node this behavior
     * is for.
     *
     * @param myNode         non-null Node to which this behavior belongs.
     * @param candidateChild non-null Node that is not equal to myNode.
     *
     * @return true if the child can be added, or false otherwise.
     */
    public boolean allowChild( Node myNode, Node candidateChild ) {
        return true;
    }

    /**
     * Returns true if this node behavior is currently allowing the Node to send to the given
     * target.
     *
     * @param myNode non-null Node to which this behavior belongs.
     * @param target non-null Node that is not equal to myNode.
     */
    public boolean allowForward( Node myNode, Node target ) {
        Confidence c = conf.get( target );
        return c == null || c.confid > threshold;
    }

    /**
     * Event to the node behavior for when a packet is received by the Node.
     *
     * @param myNode non-null Node to which this behavior belongs.
     * @param packet non-null NodePacket destined for this behavior's Node.
     */
    public void receivedPacket( Node myNode, NodePacket packet ) {
        if ( packet.getSource() != null ) {
            List<Node> toRoot = myNode.getPathToRoot( packet.getStripeId() );

            if ( toRoot.get( 0 ) != packet.getSource() )
                throw new AssertionError( "Something is wrong" );

            int confidenceChange = packet.isLive() ? 1 : -1;
            for ( Node node : toRoot ) {
                changeConfidence( node, confidenceChange );
            }

        } //else we are a tree root, so no detection for now
    }

    /**
     * Event called when a Node sends a packet to a child.
     *
     * @param myNode      non-null Node to which this behavior belongs.
     * @param destination non-null Node where the packet is going.
     * @param packet      non-null NodePacket destined for this behavior's Node.
     */
    public void sentPacket( Node myNode, Node destination, NodePacket packet ) {
    }

    /**
     * Returns true if this behavior has confidence information on any Node
     */
    public boolean hasConfidenceInformation() {
        return true;
    }

    /**
     * Returns true if this behavior has debt levels for any Node
     */
    public boolean hasDebtLevelInformation() {
        return false;
    }

    /**
     * Returns this behavior's confidence of the given Node, or null if there is no opinion.
     *
     * @param n non-null Node to get information about
     *
     * @return nullable Double
     */
    public Double getConfidence( Node n ) {
        final Confidence confidence = conf.get( n );
        return ( confidence != null ) ? (double) confidence.confid : null;
    }

    /**
     * Returns this behavior's debt level measurement of the given Node, or null if there is no
     * opinion.
     *
     * @param n non-null Node to get information about
     *
     * @return nullable Double
     */
    public Double getDebtLevel( Node n ) {
        return null;
    }

    /**
     *  Changes the confidence value for a given node
     * @param node
     * @param change
     */
    private void changeConfidence( Node node, int change ) {
        Confidence c = conf.get( node );
        if ( c == null ) {
            c = new Confidence();
            conf.put( node, c );
        }
        c.confid += change;
    }
    /**
     *  Sets the confidence to zero initially
     */
    private static class Confidence {
        private int confid = 0;
    }
}
