package Fairness.Nodes;

/**
 * NodePacket represents packets in the fairness simulation. It is intended that NodePackets are
 * always routed and passed to children nodes even if in the simulated network they are not "sent"
 * at all or even "lost". This is the reason for the existance of the lost and send flags. If a
 * NodePacket arrives successfully at its destination it is considered to be a live packet.
 *
 * @author Jason Winnebeck
 */
public class NodePacket {
    private final Node source;

    private final int stripeId;

    private final boolean lost;
    private final boolean sent;

    private final int size;

    /**
     * Constructs a new NodePacket
     *
     * @param source   nullable Node as source. null means the publisher is the source
     * @param stripeId stripe ID for the data in this packet
     * @param lost     true if packet has been lost due to network errors
     * @param sent     true if the source chose to send the packet purposefully
     * @param size     size of the packet in bytes
     */
    public NodePacket( Node source, int stripeId, boolean lost, boolean sent, int size ) {
        this.source = source;
        this.stripeId = stripeId;
        this.lost = lost;
        this.sent = sent;
        this.size = size;
    }

    /**
     * Returns the source of this packet, or null if the source is the publisher.
     *
     * @return nullable Node
     */
    public Node getSource() {
        return source;
    }

    /**
     * Returns the stripe ID for this packet
     */
    public int getStripeId() {
        return stripeId;
    }

    /**
     * Returns true if this packet is being simulated as a lost packet.
     */
    public boolean isLost() {
        return lost;
    }

    /**
     * Returns true if this packet is being simulated as having been purposefully sent by the
     * source.
     */
    public boolean isSent() {
        return sent;
    }

    /**
     * Returns this packet's size in bytes.
     *
     * @return packet's size in bytes.
     */
    public int getSize() {
        return size;
    }

    /**
     * Returns true if this packet was not lost and was sent.
     */
    public boolean isLive() {
        return !lost && sent;
    }
}
