package Fairness;

import Fairness.Nodes.Node;
import Fairness.Nodes.NodeBehaviors.NodeBehavior;
import Fairness.Nodes.TreeBehaviors.NormalTreeBehavior;
import Fairness.Trees.TreeManager;
import org.gillius.Util.CheckUtils;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;

/**
 * SimulationBuilder builds a Simulation from the input file and returns a SimulationRunner that is
 * used to actually execute the Simulation. The SimulationBuilder cannot be instantiated; it has
 * only a single static method.
 *
 * @author Jason Winnebeck
 */
public class SimulationBuilder {
    private SimulationBuilder() {
    }

    /**
     * Builds a Simulation and wraps it into a SimulationRunner for execution.
     *
     * @param input     non-null input file
     * @param outputDir non-null output directory path
     *
     * @return non-null SimulationRunner
     *
     * @throws IOException if there is an error reading input files
     * @throws SimulationInputException if the input files are in the wrong format.
     */
    public static SimulationRunner buildSimulation( File input, File outputDir )
        throws IOException, SimulationInputException {
        CheckUtils.checkNull( input, "input" );
        CheckUtils.checkNull( outputDir, "outputDir" );

        if ( !input.isFile() || !outputDir.isDirectory() )
            throw new SimulationInputException( "Invalid input file or output directory path" );

        BufferedReader in = new BufferedReader( new FileReader( input ) );

        String nextLine = in.readLine();
        if ( nextLine == null )
            throw new SimulationInputException( "Must be at least 1 line in input file" );

        String[] globals = nextLine.split( " " );
        if ( globals.length != 7 )
            throw new SimulationInputException( "First line must have 7 parameters" );

        SimulationParameters params = new SimulationParameters(
            Integer.parseInt( globals[0] ),
            Integer.parseInt( globals[1] ) * 1024,
            Integer.parseInt( globals[2] ),
            Double.parseDouble( globals[3] ),
            Integer.parseInt( globals[4] )
        );

        TreeManager manager;
        try {
            manager = (TreeManager) Class.forName( globals[5] ).newInstance();
        } catch ( Exception e ) {
            throw new SimulationInputException( "Could not create tree manager", e );
        }

        double reconRate = Double.parseDouble( globals[6] );

        Simulation sim = new Simulation( params );
        sim.setName( getSimulationName( input ) );

        List<NodeMonitor> monitors = new ArrayList<NodeMonitor>();
        List<NodeBehavior> behaviors = new ArrayList<NodeBehavior>();
        Map<String, List<Node>> nodeGroups = new HashMap<String, List<Node>>();

        nextLine = in.readLine();
        while ( nextLine != null ) {
            String[] line = nextLine.split( " " );
            if ( line.length != 7 )
                throw new SimulationInputException( "Nodes line must have 7 parameters" );
            parseNodesLine( line, monitors, behaviors, nodeGroups, sim, manager );

            nextLine = in.readLine();
        }

        manager.setUp( sim );
        manager.rebuildTree();
        sim.scheduleEvent( 0.0, new PublisherSendEvent( manager ) );
        TreeReconstructionEvent reconstructor =
            new TreeReconstructionEvent( manager, monitors, reconRate );
        sim.scheduleEvent( reconRate, reconstructor );

        return new SimulationRunner( monitors, behaviors, nodeGroups, sim, outputDir );
    }

    /**
     * Create Simulation name based on input file name
     */
    private static String getSimulationName( File input ) {
        String simName = input.getName();
        int idx = simName.lastIndexOf( '.' );
        if ( idx > 0 )
            simName = simName.substring( 0, idx );
        return simName;
    }

    /**
     * Parses a single node line, creating the number of nodes it says, or throwing an exception if
     * the line does not have the correct format.
     */
    private static void parseNodesLine( String[] line, List<NodeMonitor> monitors,
                                        List<NodeBehavior> behaviors,
                                        Map<String, List<Node>> nodeGroups, Simulation sim,
                                        TreeManager manager )
        throws SimulationInputException {
        int numNodes = Integer.parseInt( line[0] );
        String groupName = line[1];
        String behaviorClassName = line[2];
        String behaviorParam = line[3];
        int inBW = Integer.parseInt( line[4] ) * 1024;
        int outBW = Integer.parseInt( line[5] ) * 1024;
        double lossRate = Double.parseDouble( line[6] );

        Class<?> behaviorClass;
        try {
            behaviorClass = Class.forName( behaviorClassName );
        } catch ( Exception e ) {
            throw new SimulationInputException(
                "Could not get node behavior class " + behaviorClassName, e );
        }

        NodeMonitor monitor = new NodeMonitor( groupName );
        monitors.add( monitor );

        List<Node> nodes = new ArrayList<Node>( numNodes );

        for ( int i = 0; i < numNodes; ++i ) {
            NodeBehavior behavior;
            try {
                behavior = (NodeBehavior) behaviorClass.newInstance();
            } catch ( Exception e ) {
                throw new SimulationInputException(
                    "Could not create node behavior " + behaviorClassName, e );
            }
            if ( !behaviorParam.equalsIgnoreCase( "null" ) )
                behavior.setParameter( behaviorParam );
            behaviors.add( behavior );

            Node n = new Node( sim, manager.getNextNodeId(), outBW, inBW, monitor, behavior,
                               new NormalTreeBehavior() );
            n.setLossRate( lossRate );
            nodes.add( n );
            manager.addNode( n );
        }

        if ( nodeGroups.containsKey( groupName ) )
            throw new SimulationInputException( "Duplicate group " + groupName + " exists" );

        nodeGroups.put( groupName, nodes );
    }
}
