package Fairness;

/**
 * SimulationParameters holds global {@link Fairness.Simulation} parameters.
 *
 * @author Jason Winnebeck
 */
public class SimulationParameters {
    private final int numStripes;
    private final int stripeBps;
    private final int packetSize;
    private final double simDuration;

    private final int randomSeed;

    /**
     * Constructs a new SimulationParameters
     *
     * @param numStripes  number of stripes in the Simulation
     * @param stripeBps   bitrate of each stripe in bits per second
     * @param packetSize  size in bytes of each stripe data packet
     * @param simDuration duration of simulation in seconds
     * @param randomSeed  random number generator seed
     */
    public SimulationParameters( int numStripes, int stripeBps, int packetSize, double simDuration,
                                 int randomSeed ) {
        if ( simDuration < 0 )
          throw new IllegalArgumentException( "Parameter simDuration to SimulationParameters.SimulationParameters must be positive or zero, but " + simDuration + " was given." );
        if ( packetSize < 0 )
          throw new IllegalArgumentException( "Parameter packetSize to SimulationParameters.SimulationParameters must be positive or zero, but " + packetSize + " was given." );
        if ( stripeBps < 0 )
          throw new IllegalArgumentException( "Parameter stripeBps to SimulationParameters.SimulationParameters must be positive or zero, but " + stripeBps + " was given." );
        if ( numStripes < 1 )
          throw new IllegalArgumentException( "Parameter numStripes to SimulationParameters.SimulationParameters must be positive, but " + numStripes + " was given." );

        this.numStripes = numStripes;
        this.stripeBps = stripeBps;
        this.packetSize = packetSize;
        this.simDuration = simDuration;
        this.randomSeed = randomSeed;
    }

    /**
     * Gets the number of stripes in the Simulation
     */
    public int getNumStripes() {
        return numStripes;
    }

    /**
     * Gets the bitrate of each stripe in bits per second
     */
    public int getStripeBps() {
        return stripeBps;
    }

    /**
     * Gets the size in bytes of each stripe data packet
     */
    public int getPacketSize() {
        return packetSize;
    }

    /**
     * Gets the duration of simulation in seconds
     */
    public double getSimDuration() {
        return simDuration;
    }

    /**
     * Gets the random number generator seed
     */
    public int getRandomSeed() {
        return randomSeed;
    }
}
