package Fairness;

import Fairness.Nodes.Node;
import Fairness.Nodes.NodeBehaviors.NodeBehavior;
import Fairness.Nodes.NodeBehaviors.SimpleNodeBehavior;
import Fairness.Nodes.TreeBehaviors.TreeBehavior;
import Fairness.Trees.SimpleTreeManager;
import Fairness.Trees.TreeManager;
import static org.junit.Assert.*;
import static org.junit.Assert.assertEquals;
import org.junit.Test;
import org.junit.Before;

import java.util.Collections;

/**
 * SimpleSimulationTest
 *
 * @author Jason Winnebeck
 */
public class SimpleSimulationTest {
    private final SimulationParameters params = new SimulationParameters( 1, 4096, 512, 100.0, 0 );

    private NodeMonitor monitor;
    private NodeBehavior nodeBehavior;
    private TreeBehavior treeBehavior;
    private Simulation sim;
    private TreeManager manager;

    @Before public void simulationSetup() {
        monitor = new NodeMonitor();

        nodeBehavior = new SimpleNodeBehavior();
        treeBehavior = new NoTreeBehavior();

        sim = new Simulation( params );

        manager = new SimpleTreeManager();
        manager.setUp( sim );
    }

    @Test public void testSingleNode() {
        createNodes( 1 );

        startSim();
        assertEquals( 1, manager.getRootNodes().size() );
        runSim();

        monitor.mark();
        assertEquals( 1, monitor.getMarks().size() );
        NodeMonitor.Mark mark = monitor.getMarks().get( 0 );

        assertEquals( 101, mark.getTotal() );
        assertEquals( 1.0, mark.getReceivedPercent(), 0.0 );
    }

    @Test public void testMultiNodes() {
        createNodes( 8 );
        startSim();
        runSim();

        monitor.mark();
        assertEquals( 1, monitor.getMarks().size() );
        NodeMonitor.Mark mark = monitor.getMarks().get( 0 );

        assertEquals( 801, mark.getTotal() );
        assertEquals( 1.0, mark.getReceivedPercent(), 0.0 );
    }

    @Test public void testWithRebuilds() {
        createNodes( 8 );
        startSim();
        TreeReconstructionEvent reconstructor =
            new TreeReconstructionEvent( manager, Collections.singletonList( monitor ), 5.0);
        sim.scheduleEvent( 5.0, reconstructor );

        runSim();

        assertEquals( 19, monitor.getMarks().size() );
        for ( NodeMonitor.Mark mark : monitor.getMarks() ) {
            assertEquals( 1.0, mark.getReceivedPercent(), 0.0 );
        }
    }

    private void createNodes( int num ) {
        for ( int i = 0; i < num; ++i ) {
            int nextNodeId = manager.getNextNodeId();
            Node n = new Node( sim, nextNodeId, 409600, 409600, monitor, nodeBehavior, treeBehavior );
            manager.addNode( n );
        }
    }

    private void runSim() {
        while ( sim.getTime() < sim.getParams().getSimDuration() ) {
            assertTrue( sim.next() );
        }
        assertTrue( sim.getTime() >= sim.getParams().getSimDuration() );
    }

    private void startSim() {
        manager.rebuildTree();
        sim.scheduleEvent( 0.0, new PublisherSendEvent( manager ) );
    }
}
